#include "pt_simulation.h"

//calculate gcd & lcm
int gcd(int x, int y){
  if (x <= 0 || y <= 0) return (0);

  while (x != y){
    if (x>y) x = x-y;
    else     y = y-x;
  }
  return (x);
}
int lcm(int x, int y){
  if (x <= 0 || y <= 0) return (0);

  long long int r = (long long int)(x/gcd(x,y))*((long long int)y);
  if (r > INT_MAX) r = INT_MAX;

  return ((int)r);
}
int ldm(int x, int y, int z){
  long long int w = ((long long int)x)*((long long int)y);
  w = w/z;
  return((int)w);
}



// for apex file (return the number of nodes)
pair<int,int> read_pre_apex(ifstream &ifs, Aton<string> &ac, map<string,int> &lt){
  char a[MAX_LEN];
  string s;
  stringstream ss;
  int x,y,z;

  int n;
  getline (ifs,s);
  sscanf(s.c_str(), "%*d %d", &n);

  int lv = 1;
  while (1){
    getline(ifs,s);
    if (s == "-1" || s.empty()) break;
    for (string::iterator it=s.begin(); it!=s.end(); it++)
      if (*it=='(' || *it==',' || *it==')') *it=' ';
    ss.str("");
    ss.clear();    
    ss << s;
    ss >> x;
    while(!ss.eof()){
      ss >> x >> y >> z;
      if (x < 0) break;
      lv = lcm(lv,z);
    }
  }

  while(1){
    getline(ifs,s);
    if (s == "-1" || s == "") break;
    sscanf(s.c_str(), "%*d %*d %s %*d %d", a, &x);
    if (ac.insert(a)){
      lt.insert(map<string,int>::value_type(a,x));
    }
    else lt[a] = lcm(lt[a],x);
  }

  ifs.clear();
  ifs.seekg(0,ios_base::beg);
  return (pair<int,int>(n,lv));
}

void read_body_apex(ifstream &ifs, int ***c, int *ci, int *cv, Aton<string> &ac, map<string,int> &lt, int lv, int trans){
  string s;
  char a[MAX_LEN];
  stringstream ss;
  int x,y,z,w,v;

  // initial state
  getline(ifs,s);
  ss << s;
  ss >> x;
  ci[x] = 1;

  // final state
  while (1){
    getline(ifs,s);
    if (s == "-1" || s.empty()) break;
    for (string::iterator it=s.begin(); it!=s.end(); it++)
      if (*it=='(' || *it==',' || *it==')') *it=' ';
    ss.str("");
    ss.clear();    
    ss << s;
    ss >> x;

    v = 0;
    while(!ss.eof()){
      ss >> y >> z >> w;
      if (y < 0) break;
      v += ldm(z,lv,w);
    }
    cv[x] += v;
  }
  // transitions
  while(1){
    getline(ifs,s);
    if (s == "-1" || s.empty()) break;
    sscanf(s.c_str(), "%d %d %s %d %d", &x,&y,a,&z,&w);
    if (trans) c[ac[a]][y][x] += ldm(z,lt[a],w);
    else c[ac[a]][x][y] += ldm(z,lt[a],w);
  }
}

// for original format file
pair<int,int> read_pre(ifstream &ifs, Aton<string> &nodes, Aton<string> &ac, map<string,int> &lt){
  char x[MAX_LEN], y[MAX_LEN], a[MAX_LEN];
  string s;
  int i;

  int li = 1;
  while(1){
    getline(ifs,s);
    if (s == "-1" || s == "") break;
    sscanf(s.c_str(), "%s %*d %d", a, &i);
    nodes.insert(a);
    li = lcm(li,i);
  }

  while(1){
    getline(ifs,s);
    if (s == "-1" || s == "") break;
    sscanf(s.c_str(), "%s %s %s %*d %d", x, y, a, &i);
    nodes.insert(x);
    nodes.insert(y);
    if (ac.insert(a)){
      lt.insert(map<string,int>::value_type(a,i));
    }
    else lt[a] = lcm(lt[a],i);
  }

  int lv = 1;
  while(1){
    getline(ifs,s);
    if (s == "-1" || s == "") break;
    sscanf(s.c_str(), "%s %*d %d", a, &i);
    nodes.insert(a);
    lv = lcm(lv,i);
  }

  ifs.clear();
  ifs.seekg(0,ios_base::beg);
  return (pair<int,int>(li,lv));
}

void read_body(ifstream &ifs, int ***c, int *ci, int *cv, Aton<string> &nodes, Aton<string> &ac, map<string,int> &lt, int li, int lv, int trans){
  string s;
  char x[MAX_LEN];
  char y[MAX_LEN];
  char a[MAX_LEN];
  int p,q;

  while(1){
    getline(ifs,s);
    if (s.empty()) break;
    sscanf(s.c_str(), "%s %d %d", x, &p, &q);
    ci[nodes[x]] += ldm(p,li,q);
  }

  while(1){
    getline(ifs,s);
    if (s.empty()) break;
    sscanf(s.c_str(), "%s %s %s %d %d", x, y, a, &p, &q);
    if (trans) c[ac[a]][nodes[y]][nodes[x]] += ldm(p,lt[a],q);
    else c[ac[a]][nodes[x]][nodes[y]] += ldm(p,lt[a],q);
  }

  while(1){
    getline(ifs,s);
    if (s.empty()) break;
    sscanf(s.c_str(), "%s %d %d", x, &p, &q);
    cv[nodes[x]] += ldm(p,lv,q);
  }
}
