#include "pt_simulation.h"

#define GLPK 2

#define NEW 0
#define APEX 2
  
int main(int argc, char **argv){
  if (argc < 3) {
    cerr << "No input file" << endl;
    return (-1);
  }

  // deal with options
  int istyle = NEW, debugptm = 0;
  int flags = 0;
  for (int i=3; i<argc; i++){
    if (strcmp(argv[i], "-f") == 0)      {flags = flags & (~BWD);}
    else if (strcmp(argv[i], "-b") == 0) {flags = flags | BWD;}
    else if (strcmp(argv[i], "-apex") == 0) {istyle = APEX;}
    else if (strcmp(argv[i], "-debugptm") == 0) {debugptm = 1;}
    else if (strcmp(argv[i], "-output") == 0) {flags = flags | OUTPUT;}
    else if (strcmp(argv[i], "-sumunder1") == 0) {flags = flags | SUMUNDERONE;}
    else if (strcmp(argv[i], "-interior") == 0) {flags = flags | INTERIOR;}
    else cerr << "Unknown option : " << argv[i] << endl;
  }

  Aton<string> xnodes;     // Nodes of X
  Aton<string> ynodes;     // Nodes of Y
  Aton<string> ac;         // Ac
  map<string, int> lt;     // from A to lcm
  int li = 1;              // lcm of ci and di
  int lv;                  // lcm of cv and dv
  int n, m, a;             // Size of X, Y, AP

  ifstream ifs1, ifs2;
  ifs1.open(argv[1]);
  if (!ifs1){
    cerr << "Error: cannot open file " << argv[1] <<endl;
    return (-1);
  }
  ifs2.open(argv[2]);
  if (!ifs2){
    cerr << "Error: cannot open file " << argv[2] <<endl;
    return (-1);
  }
  
  if (istyle == APEX){
    pair <int,int> pa = read_pre_apex(ifs1, ac, lt);
    pair <int,int> pb = read_pre_apex(ifs2, ac, lt);
    n = pa.first;
    m = pb.first;
    lv = lcm(pa.second,pb.second);
  }
  else {
    pair <int,int> pa = read_pre(ifs1, xnodes, ac, lt);
    pair <int,int> pb = read_pre(ifs2, ynodes, ac, lt);
    n = xnodes.size();
    m = ynodes.size();
    li = lcm(pa.first, pb.first);
    lv = lcm(pa.second,pb.second);
  }

  a = ac.size();

  int ***c, ***d;
  int *cv, *dv, *ci, *di;

  c  = new int** [a];
  d  = new int** [a];
  ci = new int [n];
  di = new int [m];
  cv = new int [n];
  dv = new int [m];
  int i,j,k;
  for (i=0; i<a; i++){
    c[i] = new int* [n];
    for (j=0; j<n; j++){
      c[i][j] = new int [n];
      for (k=0; k<n; k++)
	c[i][j][k] = 0;
    }
    d[i] = new int* [m];
    for (j=0; j<m; j++){      
      d[i][j] = new int [m];
      for (k=0; k<m; k++)
	d[i][j][k] = 0;
    }
  }
  for (i=0; i<n; i++) ci[i] = 0;
  for (i=0; i<m; i++) di[i] = 0;
  for (i=0; i<n; i++) cv[i] = 0;
  for (i=0; i<m; i++) dv[i] = 0;
  
  if (istyle == APEX){
    read_body_apex(ifs1, c, ci, cv, ac, lt, lv, !(flags & BWD));
    read_body_apex(ifs2, d, di, dv, ac, lt, lv, (flags & BWD));
  }
  else{
    read_body(ifs1, c, ci, cv, xnodes, ac, lt, li, lv, !(flags & BWD));
    read_body(ifs2, d, di, dv, ynodes, ac, lt, li, lv, (flags & BWD));
  }

  ifs1.close();
  ifs2.close();

  // make output file name
  string fn, fm;
  fn = argv[1];
  fm = argv[2];
  string::size_type pos;

  if ((pos = fn.find_last_of("/")) != string::npos) fn = fn.substr(pos+1);
  if ((pos = fn.find_first_of(".")) != string::npos) fn = fn.substr(0,pos);
  if ((pos = fm.find_last_of("/")) != string::npos) fm = fm.substr(pos+1);
  if ((pos = fm.find_first_of(".")) != string::npos) fm = fm.substr(0,pos);
  fn = fn + fm;

  // for debug
  if (debugptm == 1){
    cerr << "c:" << endl;
    for (int x = 0; x < a; x++){
      for (int y = 0; y < n; y++){
	for (int z = 0; z < n; z++){
	  if (!(flags & BWD)) cerr << c[x][z][y] << " ";
	  else cerr << c[x][y][z] << " ";
	}
	cerr << endl;
      }
      cerr << endl;
    }
    cerr << "d:" << endl;
    for (int x = 0; x < a; x++){
      for (int y = 0; y < m; y++){
	for (int z = 0; z < m; z++){
	  if (flags & BWD) cerr << d[x][z][y] << " ";
	  else cerr << d[x][y][z] << " ";
	}
	cerr << endl;
      }
      cerr << endl;
    }
    cerr << "cv:" << endl;
    for (int x = 0; x < n; x++) cerr << cv[x] << " ";
    cerr << endl;
    cerr << "dv:" << endl;
    for (int x = 0; x < m; x++) cerr << dv[x] << " ";
    cerr << endl;
    cerr << "ci:" << endl; 
    for (int x = 0; x < n; x++) cerr << ci[x] << " ";
    cerr << endl;
    cerr << "di:" << endl;
    for (int x = 0; x < m; x++) cerr << di[x] << " ";
    cerr << endl;
  }

  
  ofstream ofs;
  set<int> ts;
  map<int,vector<pair<int,int> > > cd;
  cout << "States : " << n << ", " << m << " Transitions : " << a << endl;

  
  double *x;
  x = new double[n*m];
  int r = 0;
  string simtype;
  if (!(flags & BWD)) {
    simtype = "Forward Simulation";
    fm = fn + "_f.out";
    cout << simtype << " : " << endl;
    r = solve_by_glpk(a,n,m,c,cv,ci,d,dv,di,x,flags);
  }
  else{
    simtype = "Backward Simulation";
    fm = fn + "_b.out";
    cout << simtype << " : " << endl;
    r = solve_by_glpk(a,m,n,d,dv,di,c,cv,ci,x,flags);
  }
  if (r){
    if(flags & OUTPUT){
      if (!ofs) cerr << "Could not open " << fm << endl;
      ofs.open(fm.c_str());
      if (!(flags&BWD)){
        for (i=0; i<m; i++){
          for (j=0; j<n; j++) ofs << setw(11) << x[i*n+j] << " ";
          ofs << endl;
        }
      }
      else{
        for (i=0; i<n; i++){
          for (j=0; j<m; j++) ofs << setw(11) << x[i*m+j] << " ";
          ofs << endl;
        }
      }
      ofs.close();
    }
    cout << simtype << " Exists" << endl;
  }
  else cout << simtype << "does NOT Exist" << endl;
  cout << endl;
  delete [] x;

  // free memory
  for( i=0; i<a; i++){
    for( j=0; j<n; j++) delete [] c[i][j];
    for( j=0; j<m; j++) delete [] d[i][j];
    delete [] c[i];
    delete [] d[i];
  }
  delete [] cv;
  delete [] dv;

  return (0);
}
